#! /usr/bin/python

# test this shared library works exactly the same as in cpp environment
# however, any changes in DataProvider ought to be followed by an update
# of patchmap using the noise_data created in the same script

import os, sys
import subprocess
import pylab
import numpy as np
import scipy.misc
from ctypes import *

def check_md5sum(file1, file2):
   p = subprocess.Popen(["md5sum", file1], stdout=subprocess.PIPE)
   phash = p.communicate()[0].split(" ")[0]
   q = subprocess.Popen(["md5sum", file2], stdout=subprocess.PIPE)
   qhash = q.communicate()[0].split(" ")[0]
   if  phash != qhash:
      print("TEST FAILED on {}".format(file1))
      sys.exit(1)

def display(img):
   pylab.imshow(img, "jet")
   pylab.title("patch map")
   pylab.axis("off")
   pylab.show()

np.random.seed(100)

folder=os.getcwd()
so = [f for f in os.listdir(folder) if f.endswith(".so")][0]
func = CDLL(os.path.join(folder, so)).classify

c_double_p = POINTER(c_double)
func.argtypes = [c_double_p, c_int, c_int, c_double, c_int, c_int]
func.restype = np.ctypeslib.ndpointer(dtype=c_int, shape=(505, 505))

sigma=10
pSize=8

test_folder=os.path.join(folder, "test")
clean = scipy.misc.imread(os.path.join(test_folder, "lena.png"))
noise = np.random.normal(scale=sigma, size=clean.shape).T

# output noise for sanity check
noise_data = open("noise_data", "wb")
noise_data.write(bytearray(noise))

noisy = clean + noise
rows, cols = clean.shape
pmap = func(noisy.ctypes.data_as(c_double_p), rows, cols, sigma, pSize, 1)
check_md5sum("noisy.png", os.path.join(test_folder, "noisy.png"))
check_md5sum("patchmap.png", os.path.join(test_folder, "patchmap_1.png"))
pmap = func(noisy.ctypes.data_as(c_double_p), rows, cols, sigma, pSize, 2)
check_md5sum("patchmap.png", os.path.join(test_folder, "patchmap_2.png"))

# display(pmap)

os.remove("noise_data")
os.remove("noisy.png")
os.remove("patchmap.png")
